==> gabber-0.8.1.90.tar.gz: gabber-0.8.1.90/src/GabberUtility.cc <==
/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 * 
 *  Gabber
 *  Copyright (C) 1999-2001 Dave Smith & Julian Missig
 */

#include "GabberUtility.hh"
#include "Gabber.hh"
#include "e-unicode.h"
#include "ConfigManager.hh"

using namespace jabberoo;
using namespace GabberUtil;

void GabberUtil::toggle_visible(Gtk::ToggleButton* t, Gtk::Widget* w)
{
     if (t->get_active())
	  w->show();
     else
	  w->hide();
}

gint GabberUtil::strcasecmp_clist_items(GtkCList* c, const void* lhs, const void* rhs)
{
     const char* lhs_txt = GTK_CELL_PIXTEXT(((GtkCListRow*)lhs)->cell[c->sort_column])->text;
     const char* rhs_txt = GTK_CELL_PIXTEXT(((GtkCListRow*)rhs)->cell[c->sort_column])->text;
     return  strcasecmp(lhs_txt, rhs_txt);
}

void GabberUtil::change_pixmap(Gtk::PixmapMenuItem* pitem, const string& filename)
{
     // The number of incomplete widgets in gnome-libs is starting to scare me
     // This function claws inside the guts of the widget
     // I found something like this inside bonobo
     Gtk::Pixmap* pitem_pixmap = manage(new Gtk::Pixmap(filename));
     GtkPixmapMenuItem* gack = pitem->gtkobj();
     if (gack->pixmap)
     {
	  gtk_widget_destroy(gack->pixmap);
	  gack->pixmap = NULL;
     }
     pitem_pixmap->show();
     pitem->set_pixmap(*pitem_pixmap);
}

string GabberUtil::toUTF8(Gtk::Widget* w, const string& text)
{
     if (text.empty())
	  return "";
     else
     {
	 char*  value = e_utf8_from_gtk_string(w->gtkobj(), text.c_str());
	 string result = value;
	 g_free(value);
	 return result;
     }
}

string GabberUtil::toUTF8(GtkWidget* w, const string& text)
{
     if (text.empty())
	  return "";
     else
     {
	 gchar* value = e_utf8_from_gtk_string(w, text.c_str());
	 string result = value;
	 g_free(value);
	 return result;
     }
}

string GabberUtil::fromUTF8(Gtk::Widget* w, const string& text)
{
     if (text.empty())
	  return "";
     else
     {
	 gchar* value = e_utf8_to_gtk_string(w->gtkobj(), text.c_str());
	 string result = value;
	 g_free(value);
	 return result;
     }
}

string GabberUtil::fromUTF8(GtkWidget* w, const string& text)
{
     if (text.empty())
	  return "";
     else
     {
	 gchar* value = e_utf8_to_gtk_string(w, text.c_str());
	 string result = value;
	 g_free(value);
	 return result;
     }
}
string GabberUtil::getShowName(Presence::Show type)
{
     switch (type)
     {
     case Presence::stInvalid:
	  return _("Invalid");
     case Presence::stOffline:
	  return _("Offline");
     case Presence::stOnline:
	  return _("Online");
     case Presence::stChat:
	  return _("Chatty");
     case Presence::stAway:
	  return _("Away");
     case Presence::stXA:
	  return _("Extended Away");
     case Presence::stDND:
	  return _("Do Not Disturb");
     }
     return "";
}

string GabberUtil::getS10nName(Roster::Subscription type)
{

     switch (type)
     {
     case Roster::rsBoth:
	  return _("both");
     case Roster::rsFrom:
	  return _("from");
     case Roster::rsTo:
	  return _("to");
     case Roster::rsNone:
     default:
	  return _("none");
     }
}

string GabberUtil::getS10nInfo(Roster::Subscription type)
{
     switch (type)
     {
     case Roster::rsBoth:
	  return _("Both of you can see each other's presence.");
     case Roster::rsFrom:
	  return _("You cannot see their presence, but they can see yours.");
     case Roster::rsTo:
	  return _("You can see their presence, but they cannot see yours.");
	  break;
     case Roster::rsNone:
     default:
	  return _("Neither of you can see the other's presence.");
     }
}

// We have these indexShow functions in case the enum changes
Presence::Show GabberUtil::indexShow(int show_index)
{
     switch (show_index)
     {
     case 0:                 // Invalid
	  return Presence::stInvalid;
     case 1:                 // Offline
	  return Presence::stOffline;
     case 2:                 // Online
	  return Presence::stOnline;
     case 3:		     // Chat
	  return Presence::stChat;
     case 4:		     // Away
	  return Presence::stAway;
     case 5:		     // Not available
	  return Presence::stXA;
     case 6:		     // Do not disturb
	  return Presence::stDND;
     default:
	  return Presence::stInvalid;
     }
}

int GabberUtil::indexShow(Presence::Show show_index)
{
     switch (show_index)
     {
     case Presence::stInvalid:
	  return 0;                 // Invalid
     case Presence::stOffline:
	  return 1;                 // Offline
     case Presence::stOnline:
	  return 2;                 // Online
     case Presence::stChat:
	  return 3;		     // Chat
     case Presence::stAway:
	  return 4;		     // Away
     case Presence::stXA:
	  return 5;		     // Not available
     case Presence::stDND:
	  return 6;		     // Do not disturb
     default:
	  return 0;
     }
}

// Menu Builder - easy way to create an option menu with signals
/*
 *  The code in this class is
 *  Copyright (c) 2000 ERDI Gergo
 *  Also under the GPL
 */

MenuBuilder::MenuBuilder()
     : menu(manage(new Gtk::Menu()))
{
}

void MenuBuilder::add_tearoff()
{
     Gtk::TearoffMenuItem* tmi = manage(new Gtk::TearoffMenuItem());
     menu->add(*tmi);
     tmi->show();
}

void MenuBuilder::add_separator()
{
     Gtk::MenuItem* item = manage(new Gtk::MenuItem());

     // Add to menu
     menu->add(*item);
     item->show();
}

Gtk::MenuItem* MenuBuilder::add_item(const string& label, int num, bool submenu)
{
     // Ugh. Ok, so here's the breakdown:
     // label - the label to appear on the menu item
     // num - the number to send through the selected signal, usually this is the placement in the menu
     // submenu - whether or not this item will have a submenu; nasty things happen if you connect activate then

     // Create the menuitem, which has a nice constructor
     Gtk::MenuItem* item = manage(new Gtk::MenuItem(label));
     if (!submenu) // If this is not an item which will have a submenu
	  item->activate.connect(SigC::bind(selected.slot(), num));

     // Add to menu
     menu->add(*item);
     item->show();

     return item;
}

Gtk::PixmapMenuItem* MenuBuilder::add_item(const string& pixmap, const string& label, int num, bool submenu)
{
     // Ugh. Ok, so here's the breakdown:
     // pixmap - the filename of the pixmap we want to appear
     // label - the label to appear on the menu item
     // num - the number to send through the selected signal, usually this is the placement in the menu
     // submenu - whether or not this item will have a submenu; nasty things happen if you connect activate then

     // Create a pixmapmenuitem, pixmap, and label
     Gtk::PixmapMenuItem* pitem = manage(new Gtk::PixmapMenuItem());
     Gtk::Label* pitem_label = manage(new Gtk::Label(label, 0.0, 0.5));
     Gtk::Pixmap* pitem_pixmap = manage(new Gtk::Pixmap(pixmap));

     // Set the pixmap
     pitem->set_pixmap(*pitem_pixmap);
     pitem_pixmap->show();
     // Add the label
     pitem->add(*pitem_label);
     pitem_label->show();

     // Connect it
     if (!submenu) // If this is not an item which will have a submenu
	  pitem->activate.connect(SigC::bind(selected.slot(), num));

     // Add to menu
     menu->add(*pitem);
     pitem->show();

     return pitem;
}

void MenuBuilder::finish_items()
{
     menu->show();
     menu->show_all();
}

PresenceMenuBuilder::PresenceMenuBuilder()
     : _menu(_main.get_menu())
{
     // Get the path to the pixmaps
     string pix_path = ConfigManager::get_PIXPATH();

     // Add the menu items
     _main.add_tearoff();
     Gtk::PixmapMenuItem* pi;
     pi = _main.add_item(pix_path + "online.xpm", getShowName(Presence::stOnline), indexShow(Presence::stOnline), false);

     pi = _main.add_item(pix_path + "chat.xpm", getShowName(Presence::stChat), indexShow(Presence::stChat), false);

     pi = _main.add_item(pix_path + "away.xpm", getShowName(Presence::stAway), indexShow(Presence::stAway), false);

     pi = _main.add_item(pix_path + "xa.xpm", getShowName(Presence::stXA), indexShow(Presence::stXA), false);

     pi = _main.add_item(pix_path + "dnd.xpm", getShowName(Presence::stDND), indexShow(Presence::stDND), false);

     _main.finish_items();
     _main.selected.connect(main_selected.slot());
}

// File Selection and saving.
// This assumes you have something you want to save to some file

FileSel::FileSel(const string& jid, const string& title, const string& log)
     : _log(log)
{
     // Create and connect
     _fileSel = manage(new Gtk::FileSelection(title));
     _fileSel->get_ok_button()->clicked.connect(slot(this, &FileSel::on_ok_clicked));
     _fileSel->get_cancel_button()->clicked.connect(slot(this, &FileSel::on_cancel_clicked));
     
     // Get the JID and replace the @ w/ an underscore
     string basejid = JID::getUserHost(jid);
     basejid = basejid.replace(basejid.find("@"), 1, "_");
     basejid += ".log";
     
     // Setup
     _fileSel->set_filename(basejid);
     
     // Display
     _fileSel->show();
}

void FileSel::on_ok_clicked()
{
      string filename = _fileSel->get_filename();
     
      // Save the file
      ofstream file(filename.c_str());
      if (!file)
      {
	   Gnome::Dialog* d = manage(Gnome::Dialogs::warning(_("Unable to save file. Please try another name or location.")));
	   d->set_modal(true);
      }
      else
      {
	   file << _log.c_str() << endl;
	   // Get rid of file selection dialog
	   _fileSel->hide();
	   _fileSel->destroy();
      }
}

void FileSel::on_cancel_clicked()
{
     // Get rid of file selection dialog
     _fileSel->hide();
     _fileSel->destroy();
}

==> gabber-0.8.2pre1.tar.gz: gabber-0.8.2pre1/src/GabberUtility.cc <==
/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 * 
 *  Gabber
 *  Copyright (C) 1999-2001 Dave Smith & Julian Missig
 */

#include "GabberUtility.hh"
#include "Gabber.hh"
#include "e-unicode.h"
#include "ConfigManager.hh"

using namespace jabberoo;

void GabberUtil::toggle_visible(Gtk::ToggleButton* t, Gtk::Widget* w)
{
     if (t->get_active())
	  w->show();
     else
	  w->hide();
}

gint GabberUtil::strcasecmp_clist_items(GtkCList* c, const void* lhs, const void* rhs)
{
     const char* lhs_txt = GTK_CELL_PIXTEXT(((GtkCListRow*)lhs)->cell[c->sort_column])->text;
     const char* rhs_txt = GTK_CELL_PIXTEXT(((GtkCListRow*)rhs)->cell[c->sort_column])->text;
     return  strcasecmp(lhs_txt, rhs_txt);
}

string GabberUtil::toUTF8(Gtk::Widget* w, const string& text)
{
     if (text.empty())
	  return "";
     else
     {
	 char*  value = e_utf8_from_gtk_string(w->gtkobj(), text.c_str());
	 string result = value;
	 g_free(value);
	 return result;
     }
}

string GabberUtil::toUTF8(GtkWidget* w, const string& text)
{
     if (text.empty())
	  return "";
     else
     {
	 gchar* value = e_utf8_from_gtk_string(w, text.c_str());
	 string result = value;
	 g_free(value);
	 return result;
     }
}

string GabberUtil::fromUTF8(Gtk::Widget* w, const string& text)
{
     if (text.empty())
	  return "";
     else
     {
	 gchar* value = e_utf8_to_gtk_string(w->gtkobj(), text.c_str());
	 string result = value;
	 g_free(value);
	 return result;
     }
}

string GabberUtil::fromUTF8(GtkWidget* w, const string& text)
{
     if (text.empty())
	  return "";
     else
     {
	 gchar* value = e_utf8_to_gtk_string(w, text.c_str());
	 string result = value;
	 g_free(value);
	 return result;
     }
}
string GabberUtil::getShowName(Presence::Show type)
{
     switch (type)
     {
     case Presence::stInvalid:
	  return _("Invalid");
     case Presence::stOffline:
	  return _("Offline");
     case Presence::stOnline:
	  return _("Online");
     case Presence::stChat:
	  return _("Chatty");
     case Presence::stAway:
	  return _("Away");
     case Presence::stXA:
	  return _("Not Available");
     case Presence::stDND:
	  return _("Do Not Disturb");
     }
     return "";
}

string GabberUtil::getS10nName(Roster::Subscription type)
{

     switch (type)
     {
     case Roster::rsBoth:
	  return _("both");
     case Roster::rsFrom:
	  return _("from");
     case Roster::rsTo:
	  return _("to");
     case Roster::rsNone:
     default:
	  return _("none");
     }
}

string GabberUtil::getS10nInfo(Roster::Subscription type)
{
     switch (type)
     {
     case Roster::rsBoth:
	  return _("Both of you can see each other's presence.");
     case Roster::rsFrom:
	  return _("You cannot see their presence, but they can see yours.");
     case Roster::rsTo:
	  return _("You can see their presence, but they cannot see yours.");
	  break;
     case Roster::rsNone:
     default:
	  return _("Neither of you can see the other's presence.");
     }
}

// We have these indexShow functions in case the enum changes
Presence::Show GabberUtil::indexShow(int show_index)
{
     switch (show_index)
     {
     case 0:                 // Invalid
	  return Presence::stInvalid;
     case 1:                 // Offline
	  return Presence::stOffline;
     case 2:                 // Online
	  return Presence::stOnline;
     case 3:		     // Chat
	  return Presence::stChat;
     case 4:		     // Away
	  return Presence::stAway;
     case 5:		     // Not available
	  return Presence::stXA;
     case 6:		     // Do not disturb
	  return Presence::stDND;
     default:
	  return Presence::stInvalid;
     }
}

int GabberUtil::indexShow(Presence::Show show_index)
{
     switch (show_index)
     {
     case Presence::stInvalid:
	  return 0;                 // Invalid
     case Presence::stOffline:
	  return 1;                 // Offline
     case Presence::stOnline:
	  return 2;                 // Online
     case Presence::stChat:
	  return 3;		     // Chat
     case Presence::stAway:
	  return 4;		     // Away
     case Presence::stXA:
	  return 5;		     // Not available
     case Presence::stDND:
	  return 6;		     // Do not disturb
     default:
	  return 0;
     }
}

// Menu Builder - easy way to create an option menu with signals
/*
 *  The code in this class is
 *  Copyright (c) 2000 ERDI Gergo
 *  Also under the GPL
 */

void MenuBuilder::add_tearoff()
{
     Gtk::TearoffMenuItem* tmi = manage(new Gtk::TearoffMenuItem());
     menu.add(*tmi);
     tmi->show();
}

void MenuBuilder::finish_items()
{
     menu.show();
     menu.show_all();
}

void MenuBuilder::add_item(const string& label, int num)
{
     Gtk::MenuItem* item = manage(new Gtk::MenuItem(label));
     item->activate.connect(SigC::bind(selected.slot(), num));

     // Add to menu
     menu.add(*item);
     item->show();
}

void MenuBuilder::add_item(const string& pixmap, const string& label, int num)
{
     // Create a pixmapmenuitem, pixmap, and label
     Gtk::PixmapMenuItem* pitem = manage(new Gtk::PixmapMenuItem());
     Gtk::Label* pitem_label = manage(new Gtk::Label(label, 0.0, 0.5));
     Gtk::Pixmap* pitem_pixmap = manage(new Gtk::Pixmap(pixmap));

     // Set the pixmap
     pitem->set_pixmap(*pitem_pixmap);
     pitem_pixmap->show();
     // Add the label
     pitem->add(*pitem_label);
     pitem_label->show();
     // Connect it and show
     pitem->activate.connect(SigC::bind(selected.slot(), num));

     // Add to menu
     menu.add(*pitem);
     pitem->show();
}

void MenuBuilder::add_showas_items(int num)
{
     // num is the number we start at
     string pix_path = ConfigManager::get_PIXPATH();
     add_item(pix_path + "online.xpm", _("Online"), num++);
     add_item(pix_path + "chat.xpm", _("Chatty"), num++);
     add_item(pix_path + "away.xpm", _("Away"), num++);
     add_item(pix_path + "xa.xpm", _("Not Available"), num++);
     add_item(pix_path + "dnd.xpm", _("Do Not Disturb"), num);
}


// File Selection and saving.
// This assumes you have something you want to save to some file

FileSel::FileSel(const string& jid, const string& title, const string& log)
     : _log(log)
{
     // Create and connect
     _fileSel = manage(new Gtk::FileSelection(title));
     _fileSel->get_ok_button()->clicked.connect(slot(this, &FileSel::on_ok_clicked));
     _fileSel->get_cancel_button()->clicked.connect(slot(this, &FileSel::on_cancel_clicked));
     
     // Get the JID and replace the @ w/ an underscore
     string basejid = JID::getUserHost(jid);
     basejid = basejid.replace(basejid.find("@"), 1, "_");
     basejid += ".log";
     
     // Setup
     _fileSel->set_filename(basejid);
     
     // Display
     _fileSel->show();
}

void FileSel::on_ok_clicked()
{
      string filename = _fileSel->get_filename();
     
      // Save the file
      ofstream file(filename.c_str());
      if (!file)
      {
	   Gnome::Dialog* d = manage(Gnome::Dialogs::warning(_("Unable to save file. Please try another name or location.")));
	   d->set_modal(true);
      }
      else
      {
	   file << _log.c_str() << endl;
	   // Get rid of file selection dialog
	   _fileSel->hide();
	   _fileSel->destroy();
      }
}

void FileSel::on_cancel_clicked()
{
     // Get rid of file selection dialog
     _fileSel->hide();
     _fileSel->destroy();
}
